@@ -1,8 +1,11 @@
 from __future__ import absolute_import, division, print_function
+
 from collections import namedtuple
 import json
 import multiprocessing.pool
+import os
 import re
+import sys
 
 import requests
 
@@ -33,11 +36,13 @@ _urls = {
     },
     'espn': {
         'owner': 'http://games.espn.go.com/ffl/leaguesetup'
-                 '/ownerinfo?leagueId=%s',
+                 '/ownerinfo?leagueId={league_id}&seasonId={season_id}',
         'matchup': 'http://games.espn.go.com/ffl/scoreboard?'
-                   'leagueId=%s&matchupPeriodId=%d',
+                   'leagueId={league_id}&matchupPeriodId={week}'
+                   '&seasonId={season_id}',
         'roster': 'http://games.espn.go.com/ffl/playertable/prebuilt/'
-                  'manageroster?leagueId=%s&teamId=%s&scoringPeriodId=%d'
+                  'manageroster?leagueId={league_id}&teamId={team_id}'
+                  '&seasonId={season_id}&scoringPeriodId={week}'
                   '&view=overview&context=clubhouse'
                   '&ajaxPath=playertable/prebuilt/manageroster'
                   '&managingIr=false&droppingPlayers=false&asLM=false',
@@ -49,6 +54,12 @@ def pp(soup):
     print(soup.prettify().encode('utf-8'))
 
 
+def eprint(*args, **kwargs):
+    kwargs['file'] = sys.stderr
+    args = ['[nflfan]'] + list(args)
+    print(*args, **kwargs)
+
+
 def player_search(db, full_name, team=None, position=None):
     """
     A thin wrapper around `nfldb.player_search` that tries searching
@@ -72,10 +83,13 @@ def player_search(db, full_name, team=None, position=None):
 
 
 class League (namedtuple('League',
-                         'season ident prov_name name scoring conf')):
+                         'season phase ident prov_name name scoring conf')):
     __pdoc__['League.season'] = \
         """The year of the NFL season for this league."""
 
+    __pdoc__['League.phase'] = \
+        """The phase of the season: preseason, regular or post."""
+
     __pdoc__['League.ident'] = \
         """
         A unique identifier for this league. The type and format of
@@ -89,7 +103,7 @@ class League (namedtuple('League',
         """The name of this league from the configuration."""
 
     __pdoc__['League.scoring'] = \
-        """The `nflfan.ScoreScheme` for this league."""
+        """The `nflfan.ScoreSchema` for this league."""
 
     __pdoc__['League.conf'] = \
         """
@@ -125,14 +139,34 @@ class League (namedtuple('League',
     def owners(self, week):
         return self._cached(week, 'owners')
 
+    def owner(self, week, ident):
+        for o in self.owners(week):
+            if o.ident == ident:
+                return o
+        return None
+
     def matchups(self, week):
         return self._cached(week, 'matchups')
 
+    def matchup(self, week, ident):
+        for m in self.matchups(week):
+            if m.owner1.ident == ident or m.owner2.ident == ident:
+                return m
+        return None
+
     def rosters(self, week):
         return self._cached(week, 'rosters')
 
+    def roster(self, week, ident):
+        for r in self.rosters(week):
+            if r.owner.ident == ident:
+                return r
+        return None
+
     def cache_path(self, week):
-        return nflfan.config.json_path(self.full_name + ('.%d' % week))
+        return os.path.join(nflfan.config.cache_path(),
+                            str(self.season), str(self.phase), str(week),
+                            self.full_name + '.json')
 
     def _cached(self, week, key):
         if week not in self._cache:
@@ -166,6 +200,9 @@ class League (namedtuple('League',
             d['rosters'].append(r)
         self._cache[week] = d
 
+    def __str__(self):
+        return self.full_name
+
 
 class Matchup (namedtuple('Matchup', 'owner1 owner2')):
     __pdoc__['Matchup.owner1'] = \
@@ -180,6 +217,17 @@ class Matchup (namedtuple('Matchup', 'owner1 owner2')):
         `nflfan.Owner` object.
         """
 
+    def other(self, ident):
+        """
+        Given an identifier for one of the owner's in this matchup,
+        return the `nflfan.Owner` of the other owner.
+        """
+        assert ident in (self.owner1.ident, self.owner2.ident)
+        if ident == self.owner1.ident:
+            return self.owner2
+        else:
+            return self.owner1
+
     def __str__(self):
         return '%s vs. %s' % (self.owner1, self.owner2)
 
@@ -338,7 +386,7 @@ class Provider (object):
     provider_name = None
     """The name of the provider used in the configuration file."""
 
-    conf_required = ['scoring', 'league_name', 'season', 'league_id']
+    conf_required = ['scoring', 'league_name', 'season', 'phase', 'league_id']
     """A list of fields required for every provider."""
 
     conf_optional = ['me']
@@ -398,11 +446,13 @@ class Provider (object):
         def roster(owner):
             return self.roster(player_search, owner, week)
 
-        pool = multiprocessing.pool.ThreadPool(3)
-        d['rosters'] = pool.map(roster, d['owners'])
+        # pool = multiprocessing.pool.ThreadPool(3) 
+        # d['rosters'] = pool.map(roster, d['owners']) 
+        d['rosters'] = map(roster, d['owners'])
         json.dump(d, open(fp, 'w+'))
 
     def _request(self, url):
+        eprint('download %s' % url)
         r = self._session.get(url)
         soup = BeautifulSoup(r.text)
         if self._login_form(soup):
@@ -435,6 +485,9 @@ class Provider (object):
     def _login_form(self, soup):
         assert False, 'subclass responsibility'
 
+    def __str__(self):
+        return self.__class__.provider_name
+
 
 class Yahoo (Provider):
     provider_name = 'yahoo'
@@ -542,7 +595,8 @@ class ESPN (Provider):
     _login_url = 'http://games.espn.go.com/ffl/signin?_=_'
 
     def owners(self):
-        url = _urls['espn']['owner'] % self._lg.ident
+        url = _urls['espn']['owner'].format(
+            league_id=self._lg.ident, season_id=self._lg.season)
         soup = BeautifulSoup(self._request(url).text)
         owners = []
         for td in soup.select('tr.ownerRow td.teamName'):
@@ -553,7 +607,8 @@ class ESPN (Provider):
     def matchups(self, week):
         owner_id = self._owner_id_from_url
 
-        url = _urls['espn']['matchup'] % (self._lg.ident, week)
+        url = _urls['espn']['matchup'].format(
+            league_id=self._lg.ident, season_id=self._lg.season, week=week)
         soup = BeautifulSoup(self._request(url).text)
         matchupDiv = soup.find(id='scoreboardMatchups')
         matchups = []
@@ -607,7 +662,9 @@ class ESPN (Provider):
                 player = player_search(name, team=team, position=pos)
                 return r.new_player(pos, team, bench, player.player_id)
 
-        url = _urls['espn']['roster'] % (self._lg.ident, owner.ident, week)
+        url = _urls['espn']['roster'].format(
+            league_id=self._lg.ident, season_id=self._lg.season, week=week,
+            team_id=owner.ident)
         soup = BeautifulSoup(self._request(url).text)
 
         roster = Roster(owner, self._lg.season, week, [])
